import json
from pi_json import _encode_classes, PiJsonEncoder
from test.clz import Example1, Example


if __name__ == '__main__':
  ex2 = Example1("address1", True)
  ex1 = Example1("address", False, ex2)
  ex = Example(ex1, "test", 10)
  print(_encode_classes)
  jex = json.dumps(ex, indent=2, cls=PiJsonEncoder)
  print(jex)



from pi_json import json_encode


@json_encode(mapping="to_dict")
class Example:
    def __init__(self,ex, name="", value=10):
        self.name = name
        self.value = value
        self.other = ex

    def to_dict(self):
        return {
            "name": self.name,
            "value": self.value,
            "other": self.other
        }


@json_encode(mapping="to_dict")
class Example1:

    def __init__(self, address="", ignored=False, ex=None):
        self.address = address
        self.ignored = ignored
        self.example = ex

    def to_dict(self):
        return {
            "address": self.address,
            "ignored": self.ignored,
            "example": self.example
        }


import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()
VERSION = (HERE / "__init__.py").read_text()

# This call to setup() does all the work
setup(
    name="pipi_json",
    version=VERSION,
    description="JSON supported",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/eronrunner/pipi_json",
    author="Eron",
    author_email="eron.runner@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=("test",)),
    include_package_data=True,
    install_requires=[],
    entry_points={},
)


import json

class PiJsonEncoder(json.JSONEncoder):

  def default(self, o):
    for cls, f_ in _encode_classes.items():
      if isinstance(o, cls):
        if callable(f_):
          return f_()
        elif hasattr(o, f_):
          return getattr(o, f_)()
        else:
          return f_
    return json.JSONEncoder.default(self, o)

def json_encode(_cls=None, *, mapping=None):
  def decorator_encode(cls):
    if cls and mapping:
      _encode_classes.update({cls: mapping})
    return cls

  if _cls is None:
    return decorator_encode
  else:
    return decorator_encode(_cls)

_default_json_encoder = PiJsonEncoder
_encode_classes = {}
